@@ -26,45 +26,76 @@ sub clear {
     return;
 }
 
-sub update_all {
-    my $cv = AE::cv;
+sub update_all ($cb = undef) {
+    my $blocking_cv = defined wantarray ? AE::cv : undef;
+
+    my $success_all = 1;
+
+    my $cv = AE::cv sub {
+        $cb->($success_all) if $cb;
+
+        $blocking_cv->($success_all) if $blocking_cv;
+
+        return;
+    };
+
+    $cv->begin;
 
     for ( keys $RES->%* ) {
-        update( $_, $cv );
+        $cv->begin;
+
+        update(
+            $_,
+            sub ($success) {
+                $success_all = 0 if !$success;
+
+                $cv->end;
+
+                return;
+            }
+        );
     }
 
-    $cv->recv;
+    $cv->end;
 
-    return;
+    return $blocking_cv ? $blocking_cv->recv : ();
 }
 
-sub update ( $type, $cv = undef ) {
+sub update ( $type, $cb = undef ) {
     state $init = !!require IO::Uncompress::Gunzip;
 
-    $cv->begin if $cv;
+    my $blocking_cv = defined wantarray ? AE::cv : undef;
 
     P->http->get(
         $RES->{$type}->[1],
         buf_size    => 1,
         on_progress => 1,
         on_finish   => sub ($res) {
+            my $success = 0;
+
             if ( $res->status == 200 ) {
-                my $temp = P->file->tempfile;
+                eval {
+                    my $temp = P->file->tempfile;
 
-                IO::Uncompress::Gunzip::gunzip( $res->body, $temp->path, BinModeOut => 1 );
+                    IO::Uncompress::Gunzip::gunzip( $res->body, $temp->path, BinModeOut => 1 );
 
-                $ENV->share->store( $RES->{$type}->[0], $temp->path, 'Pcore-GeoIP' );
+                    $ENV->share->store( $RES->{$type}->[0], $temp->path, 'Pcore-GeoIP' );
 
-                delete $H->{$type};
+                    delete $H->{$type};
+
+                    $success = 1;
+                };
             }
 
-            $cv->end if $cv;
+            $cb->($success) if $cb;
+
+            $blocking_cv->($success) if $blocking_cv;
 
             return;
         }
     );
 
-    return;
+    return $blocking_cv ? $blocking_cv->recv : ();
 }
 
 sub country {
@@ -132,7 +163,9 @@ sub _get_h ($type) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 32                   | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
+## |    3 | 44                   | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
+## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
+## |    3 | 77                   | ErrorHandling::RequireCheckingReturnValueOfEval - Return value of eval not tested                              |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
